@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2000-2011 The OpenLDAP Foundation.
+ * Copyright 2000-2012 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -413,11 +413,8 @@ mdb_idl_insert_keys(
 
 	assert( id != NOID );
 
-	if ( slapMode & SLAP_TOOL_QUICK )
-		flag |= MDB_APPEND;
-
 #ifndef MISALIGNED_OK
-	if (keys[0].bv_len & 0x03)
+	if (keys[0].bv_len & ALIGNER)
 		kbuf[1] = 0;
 #endif
 	for ( k=0; keys[k].bv_val; k++ ) {
@@ -425,7 +422,7 @@ mdb_idl_insert_keys(
 	 * exists and if it's a range.
 	 */
 #ifndef MISALIGNED_OK
-	if (keys[k].bv_len & 0x03) {
+	if (keys[k].bv_len & ALIGNER) {
 		key.mv_size = sizeof(kbuf);
 		key.mv_data = kbuf;
 		memcpy(key.mv_data, keys[k].bv_val, keys[k].bv_len);
@@ -493,6 +490,8 @@ mdb_idl_insert_keys(
 				}
 			} else {
 			/* There's room, just store it */
+				if ( slapMode & SLAP_TOOL_QUICK )
+					flag |= MDB_APPEND;
 				goto put1;
 			}
 		} else {
@@ -519,6 +518,7 @@ mdb_idl_insert_keys(
 			}
 		}
 	} else if ( rc == MDB_NOTFOUND ) {
+		flag &= ~MDB_APPEND;
 put1:	data.mv_data = &id;
 		data.mv_size = sizeof(ID);
 		rc = mdb_cursor_put( cursor, &key, &data, flag );
@@ -563,7 +563,7 @@ mdb_idl_delete_keys(
 	assert( id != NOID );
 
 #ifndef MISALIGNED_OK
-	if (keys[0].bv_len & 0x03)
+	if (keys[0].bv_len & ALIGNER)
 		kbuf[1] = 0;
 #endif
 	for ( k=0; keys[k].bv_val; k++) {
@@ -571,7 +571,7 @@ mdb_idl_delete_keys(
 	 * exists and if it's a range.
 	 */
 #ifndef MISALIGNED_OK
-	if (keys[k].bv_len & 0x03) {
+	if (keys[k].bv_len & ALIGNER) {
 		key.mv_size = sizeof(kbuf);
 		key.mv_data = kbuf;
 		memcpy(key.mv_data, keys[k].bv_val, keys[k].bv_len);
